#ifndef __GLOBAL_H__
#define __GLOBAL_H__

#include "lwip/tcp.h"      //struct tcp_pcb
#include "config.h"

#ifdef XPAR_XEMAC_NUM_INSTANCES
#include "netif/xemacif.h" //XEmacIf_Config
#else
#include "netif/xemacliteif.h" //XEmacIf_Config
#endif
/*****************************************************/
/* Dclaration des variables et fonctions externes : */
/*****************************************************/

/** Configuration des interfaces rseaux du FPGA (gnr par EDK dans le fichier xemac_g.h).
 * On retrouve cette variable dans le fichier lwip_v1_00_a/src/contrib/ports/v2pro/netif/xemacif.c
 */
extern XEmacLiteIf_Config XEmacLiteIf_ConfigTable[];

/** Fonction d'affichage d'une chaine de caractres sur stdin (gnralement le port srie).
 * On retrouve cette fonction dans le fichier standalone_v1_00_a/src/print.c
 */
extern void print(char *ptr);

/** Fonction d'affichage avance sur stdin (gnralement le port srie).
 * On retrouve cette fonction dans le fichier standalone_v1_00_a/src/xil_printf.c
 */
extern void xil_printf(const  char *ctrl1, ...);

/**
 * Fonction d'affichage d'un nombre hexadcimal sur stdin (gnralement le port srie).
 * On retrouve cette fonction dans le fichier standalone_v1_00_a/src/putnum.c
 */
extern void putnum(unsigned int num);


/********************************/
/* Dclaration de nos fonctions */
/********************************/

/** Exemple d'utilisation de lwIP avec l'API raw.
 *
 * Cette fonction effectue les oprations suivantes : 
 *  - active les caches d'instructions et de donnes pour la plage d'adresse [0, 0x7FFFFFF].
 *  - initialise la pile TCP/IP de lwIP.
 *  - configure l'interface rseau de lwIP.
 *  - initialise notre serveur et/ou notre client.
 *  - lance la fonction principale : run().
 *  .
 *
 * \return -1 en cas d'erreur, 0 sinon.
 */
int main(void);

/** Initialise la pile TCP/IP de lwIP.
 *
 * Cette fonction effectue les oprations suivantes :  
 *  - initialise la couche "system architecture",
 *    cette dernire fourni les systmes de smaphores et de messages  la pile lwIP.
 *    Dans notre cas (Virtex2Pro) cette fonction ne fait rien puisque c'est la couche
 *    Xilinx (XMK) qui s'en occupe. Nous gardons cet appel par soucis de portabilit.
 *  - initialise le gestionnaire de mmoire dynamique,
 *    ce dernier fourni les fonctions mem_malloc(), mem_free(), mem_realloc() pour
 *    le fonctionnement de lwIP
 *  - initialise le gestionnaire de mmoire dynamique,
 *    ce dernier est spcifique aux protocoles rseaux grs par lwIP.
 *  - initialise le gestionnaire de paquets.
 *    ce dernier s'occupe des paquets IP entrants et sortants.
 *  - initialise la couche d'abstraction des interfaces rseaux,
 *    cette dernire permet d'associe une adresses IP, un masque de sous-rseau,
 *    une adresse de passerelle, ... ,  une interface rseaux.
 *  - initialise la couche TCP.
 *    cette dernire est une implmentation de la fameuse pile TCP/IP.
 *  .
 *
 * \return toujours 0, tant donn qu'aucune fonction d'initialisation de lwIP ne renvoie de
 *         code d'erreur.
 */
int lwip_init(void);

/** Configure l'interface rseau de lwIP.
 *
 * Cette fonction effectue les oprations suivantes :
 *  - enregistre l'adresse MAC auprs du pilote,
 *    cette fonction fait partie du portage de la pile lwIP par Xilinx.
 *  - configure l'interface rseau et initialise le priphrique grce au callback xemacif_init(), 
 *    ce dernier faisant partie du portage Xilinx de lwIP. Le callback ip_input() est une fonction
 *    fournie par lwIP, permettant de traiter les paquets IP entrants.
 *  - configure notre interface rseau comme tant celle par dfaut.
 *  .
 *
 * \param mac     adresse MAC de l'interface rseau.
 * \param ip      adresse IP du FPGA.
 * \param subnet  adresse de sous rseau.
 * \param gateway adresse de la passerelle.
 *
 * \return NULL en cas d'erreur, sinon un pointeur sur la structure reprsentant notre
 *         interface rseau.  
 */
struct netif *lwip_config(const unsigned char mac[6], const unsigned char ip[4],
		 const unsigned char subnet[4], const unsigned char gateway[4]);

#ifdef SERVER
/** Initialise notre serveur.
 *
 * Cette fonction effectue les oprations suivantes : 
 *  - alloue un bloc de contrle TCP pour lwIP.
 *  - enregistre auprs de lwIP le pointeur de server_close qui sera passe en tant que
 *    paramtre arg  toutes nos fonctions callback du serveur.
 *  - enregistre auprs de lwIP le callback server_err() qui sera appel  chaque fois
 *    qu'une erreur se produit.
 *  - initialise le port de connexion.
 *  - place le port dans l'tat d'coute.
 *  - enregistre auprs de lwIP le callback server_accept() qui sera appel  chaque
 *    acceptation de connexion,  partir de ce moment nous sommes capable d'accepter les
 *    connexions entrantes.
 *  .
 *
 * \param port port TCP d'coute du serveur.
 *
 * \return -1 en cas d'erreur, 0 sinon.
 *
 * \note PCB signifie Protocol Control Block, il s'agit d'une structure opaque utilise par lwIP.
 */
int server_init(short int port);

/** Fonction d'acceptation de connexion.
 *
 * Cette fonction enregistre simplement auprs de lwIP le callback server_recv() qui sera appel 
 * lors de la reception de donnes.
 *
 * \param arg pointeur sur nos donnes personnelles. 
 * \param pcb pointeur sur le PCB de la connexion.
 * \param err ERR_OK s'il n'y  pas eu de problme avant l'acceptation de la connexion.
 *
 * \return ERR_OK en cas de succs, sinon le code d'erreur.
 */
err_t server_accept(void *arg, struct tcp_pcb *pcb, err_t err);

/** Fonction appele lors de la rception de donnes.
 * 
 * Cette fonction effectue les oprations suivantes :
 *  - si la liste des paquets n'est pas vide (i.e. pbuf != NULL), nous renvoyons simplement
 *    les donnes reues (uniquement celles du premier paquet).
 *  - enregistre auprs de LwIP le nombre de donnes reues.
 *  - enregiste le callback server_sent() qui sera appel lorsque les donnes mise seront acquittes.
 *  - en fonction de la configuration, nous fermons la connexion, mais le serveur sera alors  nouveau en tat
 *    d'coute sur le port, prs  accepter un nouvelle connexion.
 *  .
 *
 * \param arg pointeur sur nos donnes personnelles. 
 * \param pcb pointeur sur le PCB de la connexion.
 * \param pbuf pointeur sue la liste chaine des paquets reus.
 * \param err ERR_OK s'il n'y  pas eu de problme avant l'acceptation de la connexion.
 *
 * \return ERR_OK en cas de succs, sinon le code d'erreur.
 */
err_t server_recv(void *arg, struct tcp_pcb *pcb, struct pbuf *pbuf, err_t err);

/** Fonction appele lors de l'acquittement des donnes mises.
 *
 * Cette fonction ne fait qu'afficher sur stdin le nombre d'octets envoy avec succs.
 * 
 * \param arg pointeur sur nos donnes personnelles. 
 * \param pcb pointeur sur le PCB de la connexion.
 * \param len taille des donnes mises correctement.
 *
 * \return ERR_OK en cas de succs, sinon le code d'erreur.
 */
err_t server_sent(void *arg, struct tcp_pcb *pcb, u16_t len);

/** Fonction appele lors d'une erreur.
 *
 * Cette fonction fait simplement appel  print_err().
 *
 * \param arg pointeur sur nos donnes personnelles. 
 * \param err erreur qui s'est produite.
 */
void server_err(void *arg, err_t err);
#endif //SERVER

#ifdef CLIENT
/** Initialise notre client.
 *
 * Cette fonction effectue les oprations suivantes : 
 *  - alloue un bloc de contrle TCP pour lwIP.
 *  - enregistre auprs de lwIP le pointeur de client_close qui sera passe en tant que
 *    paramtre arg  toutes nos fonctions callback du serveur.
 *  - enregistre auprs de lwIP le callback client_err() qui sera appel  chaque fois
 *    qu'une erreur se produit.
 *  - initialise la connexion et enregistre le callback client_connected() qui sera appel
 *    lorsque la connexion sera tablie.
 *  .
 *
 * \param remote_ip adresse du serveur sur lequel se connecter.
 * \param remote_port numro de port du serveur sur lequel se connecter.
 *
 * \return -1 en cas d'erreur, 0 sinon.
 */
int client_init(const unsigned char remote_ip[4], short int remote_port);

/** Fonction appele lorsqu'une connexion  t tablie.
 *
 * Cette fonction effectue les oprations suivantes : 
 *  - enregistre auprs de lwIP le callback client_recv() qui sera appel  chaque fois
 *    que le client recevra des donnes.
 *  - amorce le timer qui appelera le callback client_poll() qui sera appel toutes les 
 *    POLL_INTERVAL secondes.
 *  .
 *
 * \return ERR_OK en cas de succs, sinon le code d'erreur.
 */
err_t client_connected(void *arg, struct tcp_pcb *pcb, err_t err);

/** Fonction appele lors de l'acquittement des donnes mises.
 *
 * Cette fonction ne fait qu'afficher sur stdin le nombre d'octets envoy avec succs.
 * 
 * \param arg pointeur sur nos donnes personnelles. 
 * \param pcb pointeur sur le PCB de la connexion.
 * \param len taille des donnes mises correctement.
 *
 * \return ERR_OK en cas de succs, sinon le code d'erreur.
 */
err_t client_sent(void *arg, struct tcp_pcb *pcb, u16_t len);

/** Fonction appele rgulirement pour envoy des donnes.
 *
 * Cette fonction effectue les oprations suivantes : 
 *  - initialise un paquet.
 *  - enregistre le nom de l'entreprise "SMP" dans la charge du paquet.
 *  - envoie le paquet.
 *  - enregistre auprs de lwIP le callback client_sent() qui sera appel lorsque les donnes mise seront acquittes.
 *  - libre la mmoire de ce paquet (le contenu est copier dans l'appel  tcp_write, cf le troisme paramtre).
 *  - en fontion de la configuration, ferme la connexion.
 *  .
 *
 * \return ERR_OK en cas de succs, sinon le code d'erreur.
 */
err_t client_poll(void *arg, struct tcp_pcb *pcb);

/** Fonction appele lors d'une erreur.
 *
 * Cette fonction fait simplement appel  print_err().
 *
 * \param arg pointeur sur nos donnes personnelles. 
 * \param err erreur qui s'est produite.
 */
void client_err(void *arg, err_t err);

/** Fonction appele lors de la rception de donnes.
 *
 * Cette fonction effectue les oprations suivantes :
 *  - affiche le nombre de donnes reues.
 *  - enregistre auprs de LwIP le nombre de donnes reues.
 *  - libre le paquet reu.
 *  .
 *
 * \return ERR_OK en cas de succs, sinon le code d'erreur.
 */
err_t client_recv(void *arg, struct tcp_pcb *pcb, struct pbuf *pbuf, err_t err);

#endif //CLIENT

/** Fonction principale de l'application.
 *
 * Il s'agit de la fonction la plus complique de l'application, alors des explications semblent
 * les bienvenues...
 *
 * Pour que la pile TCP/IP de lwIP fonctionne, il faut appeler certaines fonctions priodiquement.
 * Nous avons utilis le timer interne au PPC pour effectuer ces appels priodiques, grce aux
 * fonctions XTime_SetTime() et XTime_GetTime() qui, respectivment, met  jour et lit la valeur du
 * Time Base Register. Ce dernier est sur 64 bits et est incrment de 1  chaque coup d'horloge.
 *
 * Ainsi, nous pouvons (devons) appeler la fonction tcp_tmr() toutes les 100 milli-secondes et la 
 * fonction etharp_tmr() toutes les 10 secondes. Dans les autres "priodes", nous faisons
 * appel  la fonction xemacif_input() afin que lwIP puisse lire les trames Ethernet entrant sur
 * l'interface rseau.
 *
 * \param netif interface rseau o lire les trames Eternet.
 *
 * \note TCP_TMR_INTERVAL est la priode (en milli-seconde)  laquelle tcp_tmr() doit tre appele.
 *
 * \note ARP_TMR_INTERVAL est la priode (en milli-seconde)  laquelle etharp_tmr() doit tre appele.
 *       Nous avons fait l'hypothse qu'elle est plus grande que TCP_TMR_INTERVAL.
 *
 */
void run(struct netif *netif);

/** Affiche sur stdin, de manire lisible, une erreur.
 *
 * \param err erreur  afficher.
 */
void print_err(err_t err);

#endif //__GLOBAL_H__
